;________________________________________________________________________
;
;                              M X 6 1 4 . A S M
;________________________________________________________________________
;
; Sending packet audio tones with a MX614 modem chip
; by Byon Garrabrant  N6BG  byon@mail.com  Feb 99
;________________________________________________________________________
;
;   This software is  available only for non-commercial  amateur radio 
;   or educational  applications.   ALL  other  uses  are  prohibited.  
;   This  software may  be  modified  only  if the  resulting code  be 
;   made available  publicly  and the  original  author given  credit.  
;   Contact the author or TAPR for information on commercial licenses. 
;   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved. 
;________________________________________________________________________

	cblock				; variables
		FLAGS			; generic flags / bits
		next_freq
		intWsave		; interrupt flags
		intSTATUSsave
		intPCLATHsave
	endc
;________________________________________________________________________

#define	SENT			FLAGS, 1
#define	BANK1			H'7F'
#define	TONE_OUT		PORTB, 5
#define	MODE1_OUT		PORTB, 1
#define	MODE0_OUT		PORTB, 2
;________________________________________________________________________

	ORG	H'0004'

Timer_Interrupt
	movwf	intWsave		; save W
	swapf	STATUS, W		; save STATUS
	movwf	intSTATUSsave
	movfw	PCLATH			; save PCLATH
	movwf	intPCLATHsave

	bcf	STATUS, RP0		; select bank 0 (working with TMR0)
	movlw	D'130'			; bump timer forward
	subwf	TMR0, F			; update TMR0 for next int
	bsf	SENT			; ready for the next bit

timer_end
	bcf	INTCON, T0IF		; enable TMR0 int
	movfw	intPCLATHsave		; restore PCLATH
	movwf	PCLATH			
	swapf	intSTATUSsave, W	; restore STATUS
	movwf	STATUS
	swapf	intWsave, F		; restore W
	swapf	intWsave, W
	retfie				; return from interrupt

;________________________________________________________________________

Tone_Init
	bsf	STATUS, RP0		; Select Bank 1
	bcf	TONE_OUT		; make data an output
	bcf	MODE0_OUT		; make mode0 an output
	bcf	MODE1_OUT		; make mode1 an output
	bcf	STATUS, RP0		; Select Bank 0
	clrf	INTCON

Tone_Disable
	bcf	INTCON, T0IE		; disableTMR0int
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010111'		; divide TMR0 by 256
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bcf	INTCON, GIE		; disable ints
	bsf	MODE1_OUT		; disable audio tone
	bcf	MODE0_OUT		; disable audio tone
	bcf	TONE_OUT		; disable audio tone
	return

Tone_Enable
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010011'		; div by 16
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bsf	INTCON, T0IE		; enableTMR0int
	bsf	INTCON, GIE		; enable ints
	clrf	next_freq		; set initial freq
	bcf	MODE1_OUT		; enable audio tone
	bsf	MODE0_OUT		; enable audio tone
					; possible delay needed
	return
;________________________________________________________________________

Tone_Toggle_If_No_Carry
	btfss	SENT			; wait for last bit to be sent
	 goto	$-1

	bcf 	SENT			; next bit is ready

	skpnc				; if sending a 1
	 return				; leave

	incf	next_freq, F		; set new

	btfsc	next_freq, 0
	 bsf	TONE_OUT
	btfss	next_freq, 0
	 bcf	TONE_OUT

	return
;________________________________________________________________________
